//+------------------------------------------------------------------+
//| 10:24 AM 3/17/2019                          common_utilities.mqh |
//|                             Copyright  2019, WHRoeder@yahoo.com |
//|                                        mailto:WHRoeder@yahoo.com |
//+------------------------------------------------------------------+
//{ Typedefs
#define  COUNT    uint           //  One based.
   #define  INV_COUNT      UINT_MAX
#define  INDEX    uint           // Zero based.
   #define  INV_INDEX      UINT_MAX
#define  LOTS     double
#define  MONEY    double
#define  PRICE double            // A Price
#define     PRICE_MAX   (PRICE)EMPTY_VALUE
#define     PRICE_MIN   (PRICE)0.0
#define     CHANGE      PRICE    // Difference of two prices.
#define  SYMBOL   string
#define  TICKET   int
   #define  INV_TICKET  EMPTY
#define  MAX_DATETIME   D'3000.12.31 23:59:59'
#define  MIN_DATETIME   0
#define		SECONDS	int	// Difference of two datetimes.
//} Typedefs
enum Trade_Operation{            // Same as OP_xxx except a proper enumeration.
      TO_BUY, TO_SELL, TO_BUYLIMIT, TO_SELLLIMIT, TO_BUYSTOP, TO_SELLSTOP,
      TO_BALANCE,                         ///< (6) May occur in history pool.
      TO_CREDIT};                         ///< (7) May occur in history pool.
   const Trade_Operation   MKT_FIRST   = TO_BUY;     ///< (0) OP_BUY.
   const Trade_Operation   MKT_LAST    = TO_SELL;    ///< (1) OP_SELL.
   const Trade_Operation   PND_FIRST   = TO_BUYLIMIT;///< (2) OP_BUYLIMIT.
   const Trade_Operation   PND_LAST    = TO_SELLSTOP;///< (5) OP_SELLSTOP.
   const COUNT             MKT_COUNT   = 2;          ///< (2) [OP_BUY-OP_SELL]

string	gShortName	=WindowExpertName();
void     alert_id(string msg){
   Alert(StringFormat("%s %s %s %s", gShortName, _Symbol,
                                     as_string(PERIOD_CURRENT), msg) );
}
void     print_id(string msg){
   PrintFormat(       "%s %s %s %s", gShortName, _Symbol,
                                     as_string(PERIOD_CURRENT), msg);
}
template<typename T>
string   as_signed(T t){ return t < 0 ? string(t) : "+" + string(t); }
string   as_string(ENUM_APPLIED_PRICE p, int len=0){
   string price_XXX = EnumToString(p);       // PRICE_XXX
   return StringSubstr(price_XXX, 6, len);   // XXX
}
string   as_string(ENUM_MA_METHOD p, int len=0){
   string mode_XXX = EnumToString(p);        // MODE_XXX
   return StringSubstr(mode_XXX, 5, len);    // XXX
}
string   as_string(ENUM_TIMEFRAMES period){
   if(period == PERIOD_CURRENT)  period   = (ENUM_TIMEFRAMES) _Period;
   string   period_xxx  = EnumToString(period);                // PERIOD_XXX
   return StringSubstr(period_xxx, 7);                         // XXX
}
bool     download_history(ENUM_TIMEFRAMES period=PERIOD_CURRENT){
   return download_history(_Symbol, period);
}
bool     download_history(SYMBOL symbol, ENUM_TIMEFRAMES period=PERIOD_CURRENT){
   if(period == PERIOD_CURRENT)  period = (ENUM_TIMEFRAMES)_Period;
   ResetLastError();    datetime other = iTime(symbol, period, 0);
   if(_LastError == 0 && other != 0)   return true;
   if(_LastError != ERR_HISTORY_WILL_UPDATED
   && _LastError != ERR_NO_HISTORY_DATA
     )   PrintFormat("iTime(%s,%i) Failed: %i", symbol, period, _LastError);
   return false;
}
template <typename T>
T        MathMax(T a, T b, T c){ return MathMax(a, MathMax(b, c) );  }
template <typename T>
T        MathMin(T a, T b, T c){ return MathMin(a, MathMin(b, c) );  }
template<typename T>
void     swap(T& a, T& b){  T t=a;   a=b;  b=t;  }

double   EMA(double previous, double current, COUNT length){
   double   alpha = 2.0 / (length + 1);
   return previous + alpha * (current - previous);
}
double   PMA(double previous, double current, COUNT length, double power){
   // en.wikipedia.org/wiki/Generalized_mean#Special_cases (Power Mean)
   previous = MathPow(previous, power);
   current  = MathPow(current,  power);
      double   ma = EMA(previous, current, length);
   return MathPow(ma, 1.0 / power);
}
double   MathNearest(  double v, double to){ return to * MathRound(v / to); }
double   MathRoundDown(double v, double to){ return to * MathFloor(v / to); }
double   MathRoundUp(  double v, double to){ return to * MathCeil( v / to); }

enum termnal_Mode{  MODE_DEBUG, MODE_LIVE, MODE_VISUAL, MODE_OPTIMIZER};
termnal_Mode     get_modus_operandi(void){
   if(IS_DEBUG_MODE)          return MODE_DEBUG;   // Live but debugging.
   if(!IsTesting() )          return MODE_LIVE;
   if( IsVisualMode() )       return MODE_VISUAL;
                              return MODE_OPTIMIZER;
}

double   get_spread_max(){ // Assumes indicator "spread" is attached to chart.
      static const string  nameMax  = _Symbol + "SpreadMax";
   double   value;   GlobalVariableGet(nameMax, value);
   return   value;
}

class    ATR{
 public:
            ATR(COUNT len=14)                                           //[cTor]
                            : mSymbol(_Symbol), mTF(ENUM_TIMEFRAMES(_Period) ), 
                              mLength(len){}
            ATR(ENUM_TIMEFRAMES tf, COUNT len=14)                       // cTor
                            : mSymbol(_Symbol), mTF(tf), mLength(len){}
            ATR(SYMBOL sym, ENUM_TIMEFRAMES tf=0, COUNT len=14)         // cTor
                            : mSymbol(sym), mTF(tf), mLength(len){}
   PRICE    get(INDEX iBar){  return   iATR(mSymbol, mTF, mLength, iBar);      }
 private:
   SYMBOL               mSymbol;
   ENUM_TIMEFRAMES      mTF;

   COUNT                mLength;
}; // ATR

class    MA{
 public:
            MA(ENUM_MA_METHOD ma, COUNT len=14,                         // cTor
               ENUM_APPLIED_PRICE ap=PRICE_CLOSE, int shift=0)
                         : mSymbol(_Symbol), mTF( ENUM_TIMEFRAMES(_Period) ),
                           mAP(ap), mLength(len), mMethod(ma), mShift(shift){}
            MA(ENUM_TIMEFRAMES tf, ENUM_MA_METHOD ma, COUNT len=14,     // cTor
               ENUM_APPLIED_PRICE ap=PRICE_CLOSE, int shift=0)
                         : mSymbol(_Symbol), mTF(tf),
                           mAP(ap), mLength(len), mMethod(ma), mShift(shift){}
            MA(SYMBOL sym, ENUM_TIMEFRAMES tf, ENUM_MA_METHOD ma,       // cTor
               COUNT len=14, ENUM_APPLIED_PRICE ap=PRICE_CLOSE, int shift=0)
                         : mSymbol(sym), mTF(tf),
                           mAP(ap), mLength(len), mMethod(ma), mShift(shift){}
   PRICE    get(INDEX iBar){  return
                     iMA(mSymbol, mTF, mLength, mShift, mMethod, mAP, iBar);  }
 private:
   SYMBOL               mSymbol;
   ENUM_TIMEFRAMES      mTF;

   ENUM_APPLIED_PRICE   mAP;
   COUNT                mLength;
   ENUM_MA_METHOD       mMethod;
   int                  mShift;
}; // MA

class Ticker{
// EventSetTimer sometimes fails in OnInit, on MT4, with internal error. So
// remember to try it in OnCalculate
 public:
            Ticker(void)    : mTickerStatus(false){}               //[CTor]
 public:                                                             // Methods
   bool     enable(void){
      if(mTickerStatus != TS_ACTIVE){
         if(!EventSetTimer(1) ){
            alert_id(StringFormat("EST:%i", _LastError) );
            mTickerStatus  = TS_ERROR;
            return false;
         }
         if(mTickerStatus == TS_ERROR) alert_id("EST: Successful");
         mTickerStatus  = TS_ACTIVE;
      }
      return true;
   }
   void     disable(void){ EventKillTimer();    mTickerStatus = false;       }
 private:                                                            // Data
      enum     TckStatus{   TS_IDLE,   TS_ERROR,   TS_ACTIVE   };
   TckStatus   mTickerStatus;
}; // class Ticker
//+- common_utilities.mqh -------------------------------------------+
